# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-05-23 10:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('emailtemplates', '0002_auto_20170428_1442'),
    ]

    operations = [
        migrations.CreateModel(
            name='MassEmailMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=255, verbose_name='subject')),
                ('content', models.TextField(verbose_name='content')),
                ('date_sent', models.DateTimeField(blank=True, null=True, verbose_name='sent')),
            ],
        ),
    ]



# coding=utf-8
from __future__ import unicode_literals
from django.urls import reverse

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, redirect
from django.template import Template, Context
from django.views import View
from django.utils.translation import gettext as _

from emailtemplates.models import EmailTemplate, MassEmailMessage
from emailtemplates.registry import email_templates


class EmailPreviewView(View):

    def get_email_template(self):
        return get_object_or_404(EmailTemplate, pk=self.kwargs['pk'])

    def get_context_data(self):
        email_template = self.get_email_template()
        return email_templates.get_help_content(email_template.title)

    def get(self, request, *args, **kwargs):
        email_template = self.get_email_template()
        email_content = Template(email_template.content)
        return HttpResponse(
            email_content.render(Context(self.get_context_data())),
            content_type='text/html; charset=utf-8'
        )


email_preview_view = staff_member_required(EmailPreviewView.as_view())


class SendMassEmailView(View):

    def get_mass_email_message(self):
        return get_object_or_404(MassEmailMessage, pk=self.kwargs['pk'])

    def redirect_back(self):
        return HttpResponseRedirect(
            reverse('admin:emailtemplates_massemailmessage_change', args=(self.get_mass_email_message().pk,)),
        )

    def get(self, request, *args, **kwargs):
        mass_email_message = self.get_mass_email_message()
        if mass_email_message.sent:
            messages.success(request, _("Mass email was already sent. "
                                        "Create new mail message or force sending from shell."))
            return self.redirect_back()
        sent = mass_email_message.send()
        if sent:
            messages.success(request, _("Mass email sent successfully"))
        else:
            messages.warning(request, _("Error occurred when trying to send mass email message."))
        return self.redirect_back()


send_mass_email_view = SendMassEmailView.as_view()



# coding=utf-8
from importlib import import_module
from string import lower

from django.template.loaders import app_directories
from django.contrib.auth import get_user_model
from django.conf import settings


class SubstringMatcher(object):
    """
    Class to be used with Mock() in order not to supply full content
    of the argument (e.g. for logger).
    Based on: http://www.michaelpollmeier.com/python-mock-how-to-assert-a-substring-of-logger-output/

    Usage with this class aliased to substr:
    email_logger.warning.assert_called_with(substr("Can't find EmailTemplate object in database"))

    This would do the same, but requires exactly the same argument content:
    email_logger.warning.assert_called_with("Can't find EmailTemplate object in database, using default file template.")
    """

    def __init__(self, containing):
        self.containing = lower(containing)

    def __eq__(self, other):
        return lower(other).find(self.containing) > -1

    def __unicode__(self):
        return 'a string containing "%s"' % self.containing

    def __str__(self):
        return unicode(self).encode('utf-8')

    __repr__ = __unicode__


substr = SubstringMatcher


class TemplateSourceLoader(app_directories.Loader):
    def get_source(self, template_name):
        source, origin = self.load_template_source(template_name)
        return source


def mass_mailing_recipients():
    """
    Returns iterable of all mass email recipients.
    Default behavior will be to return list of all active users' emails.
    This can be changed by providing callback in settings return some other list of users,
    when user emails are stored in many, non default models.
    To accomplish that add constant MASS_EMAIL_RECIPIENTS to settings. It should contain path to function, e.g.
    >>> MASS_EMAIL_RECIPIENTS = 'emailtemplates.helpers.mass_mailing_recipients'

    :rtype iterable
    """
    if hasattr(settings, 'MASS_EMAIL_RECIPIENTS'):
        callback_name = settings.MASS_EMAIL_RECIPIENTS.split('.')
        module_name = '.'.join(callback_name[:-1])
        func_name = callback_name[-1]
        module = import_module(module_name)
        func = getattr(module, func_name, lambda: [])
        return func()
    User = get_user_model()
    if hasattr(User, 'is_active') and hasattr(User, 'email'):
        filtered_users = User.objects.filter(is_active=True).exclude(email__isnull=True).exclude(email__exact='')
        return filtered_users.values_list('email', flat=True).distinct()
    return []



# coding=utf-8
import logging
from django.utils.translation import ugettext_lazy as _


logger = logging.getLogger(__name__)


class AlreadyRegistered(Exception):
    pass


class NotRegistered(Exception):
    pass


class HelpContext(object):
    """
    Provides helpers methods for displaying help context keys (descriptions) and values (examples).
    """
    def __init__(self, help_context):
        self.help_context = help_context or {}

    def get_help_keys(self):
        """
        Returns dict of help_context keys (description texts used in `EmailRegistry.register()` method).
        """
        help_keys = {}
        for k, v in self.help_context.items():
            if isinstance(v, tuple):
                help_keys[k] = v[0]
            else:
                help_keys[k] = v
        return help_keys

    def get_help_values(self):
        """
        Returns dict of help_context values (example values submitted in `EmailRegistry.register()` method).
        """
        help_values = {}
        for k, v in self.help_context.items():
            if isinstance(v, tuple) and len(v) == 2:
                help_values[k] = v[1]
            else:
                help_values[k] = u"<%s>" % k
        return help_values


class RegistrationItem(object):

    def __init__(self, path, help_text=u"", help_context=None):
        self.path = path
        self.help_text = help_text
        self.help_context_obj = HelpContext(help_context)

    @property
    def help_context(self):
        return self.help_context_obj.get_help_keys()

    def _context_key(self, key):
        return u"<b>{{ %s }}</b>" % key

    def context_description(self):
        help_text_item = lambda k, v: u"%s - %s" % (self._context_key(k), v) if v else u"%s" % self._context_key(k)
        return u"<br/>".join([help_text_item(k, v) for (k, v) in sorted(self.help_context_obj.get_help_keys().items())])

    def as_form_help_text(self):
        item_help_text = _(u"<b>USAGE: %s</b>") % self.help_text if self.help_text else u""
        item_help_context = _(u"<b>CONTEXT:</b><br/>%s") % self.context_description() if self.help_context_obj.get_help_keys() else u""
        return u"<br/>".join((item_help_text, item_help_context))

    def as_form_choice(self):
        return self.path, self.path

    def get_help_content(self):
        return self.help_context_obj.get_help_values()


class EmailTemplateRegistry(object):

    def __init__(self):
        self._registry = {}

    def register(self, path, help_text=None, help_context=None):
        """
        Registers email template.

        Example usage:
            email_templates.register('hello_template.html', help_text=u'Hello template',
                help_context={'username': u'Name of user in hello expression'})

        :param path: Template file path. It will become immutable registry lookup key.
        :param help_text: Help text to describe template in admin site
        :param help_context: Dictionary of possible keys used in the context and description of their content

        `help_context` items values may be strings or tuples of two strings. If strings, then email template preview
        will use variable names to fill context, otherwise the second tuple element will become example value.

        If an email template is already registered, this will raise AlreadyRegistered.
        """
        if path in self._registry:
            raise AlreadyRegistered('The template %s is already registered' % path)
        self._registry[path] = RegistrationItem(path, help_text, help_context)
        logger.debug("Registered email template %s", path)

    def is_registered(self, path):
        return path in self._registry

    def get_registration(self, path):
        """
        Returns registration item for specified path.

        If an email template is not registered, this will raise NotRegistered.
        """
        if not self.is_registered(path):
            raise NotRegistered("Email template not registered")
        return self._registry[path]

    def get_help_text(self, path):
        return self.get_registration(path).help_text

    def get_help_context(self, path):
        return self.get_registration(path).help_context

    def get_help_content(self, path):
        return self.get_registration(path).get_help_content()

    def registration_items(self):
        return self._registry.values()

    def email_template_choices(self):
        """
        Returns list of choices that can be used in email template form field choices.
        """
        return [item.as_form_choice() for item in self.registration_items()]

    def get_form_help_text(self, path):
        """
        Returns text that can be used as form help text for creating email templates.
        """
        try:
            form_help_text = self.get_registration(path).as_form_help_text()
        except NotRegistered:
            form_help_text = u""
        return form_help_text


# Global object for singleton registry of email templates
email_templates = EmailTemplateRegistry()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-05-23 16:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('emailtemplates', '0003_auto_20180523_1027'),
    ]

    operations = [
        migrations.CreateModel(
            name='MassEmailAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('attachment_file', models.FileField(upload_to=b'', verbose_name='Attachment file')),
                ('mass_email_message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='emailtemplates.MassEmailMessage')),
            ],
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='language',
            field=models.CharField(choices=[(b'de', 'German'), (b'en', 'English')], default=b'de', max_length=10, verbose_name='language'),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='title',
            field=models.CharField(max_length=255, verbose_name='template'),
        ),
    ]



# encoding: utf-8
from django.contrib.auth import get_user_model
from django.test import TestCase, override_settings
from emailtemplates.helpers import mass_mailing_recipients


def recipients_test_function():
    return ['user@example.com', 'another@example.com']

class HelpersTest(TestCase):

    def test_mass_mailing_recipients(self):
        User = get_user_model()
        User.objects.create(username="mike", email="mike@example.com", is_active=True)
        User.objects.create(username="john", email="john@example.com", is_active=False)
        User.objects.create(username="paul", is_active=True)
        self.assertEqual(list(mass_mailing_recipients()), ["mike@example.com"])
    
    @override_settings(MASS_EMAIL_RECIPIENTS='emailtemplates.tests.test_helpers.recipients_test_function')
    def test_mass_mailing_recipients_from_settings(self):
        self.assertEqual(mass_mailing_recipients(), ['user@example.com', 'another@example.com'])



from django.conf.urls import url

from emailtemplates.views import email_preview_view, send_mass_email_view

urlpatterns = [
    url(r'^email-preview/(?P<pk>\d+)/$', email_preview_view, name='email_preview'),
    url(r'^send-mass-email/(?P<pk>\d+)/$', send_mass_email_view, name='send_mass_email'),
]
